@@ -1,3 +1,6 @@
+/*
+ * $PostgreSQL: pgsql/src/interfaces/ecpg/pgtypeslib/timestamp.c,v 1.45 2009/06/11 14:49:13 momjian Exp $
+ */
 #include "postgres_fe.h"
 
 #include <time.h>
@@ -73,8 +76,9 @@ tm2timestamp(struct tm * tm, fsec_t fsec, int *tzp, timestamp * result)
 	if ((*result - time) / USECS_PER_DAY != dDate)
 		return -1;
 	/* check for just-barely overflow (okay except time-of-day wraps) */
-	if ((*result < 0 && dDate >= 0) ||
-		(*result >= 0 && dDate < 0))
+	/* caution: we want to allow 1999-12-31 24:00:00 */
+	if ((*result < 0 && dDate > 0) ||
+		(*result > 0 && dDate < -1))
 		return -1;
 #else
 	*result = dDate * SECS_PER_DAY + time;
@@ -88,45 +92,22 @@ tm2timestamp(struct tm * tm, fsec_t fsec, int *tzp, timestamp * result)
 static timestamp
 SetEpochTimestamp(void)
 {
+#ifdef HAVE_INT64_TIMESTAMP
+	int64		noresult = 0;
+#else
+	double		noresult = 0.0;
+#endif
 	timestamp	dt;
 	struct tm	tt,
 			   *tm = &tt;
 
-	GetEpochTime(tm);
+	if (GetEpochTime(tm) < 0)
+		return noresult;
+
 	tm2timestamp(tm, 0, NULL, &dt);
 	return dt;
 }	/* SetEpochTimestamp() */
 
-static void
-dt2time(timestamp jd, int *hour, int *min, int *sec, fsec_t *fsec)
-{
-#ifdef HAVE_INT64_TIMESTAMP
-	int64		time;
-#else
-	double		time;
-#endif
-
-	time = jd;
-
-#ifdef HAVE_INT64_TIMESTAMP
-	*hour = time / USECS_PER_HOUR;
-	time -= (*hour) * USECS_PER_HOUR;
-	*min = time / USECS_PER_MINUTE;
-	time -= (*min) * USECS_PER_MINUTE;
-	*sec = time / USECS_PER_SEC;
-	*fsec = time - *sec * USECS_PER_SEC;
-	*sec = time / USECS_PER_SEC;
-	*fsec = time - *sec * USECS_PER_SEC;
-#else
-	*hour = time / SECS_PER_HOUR;
-	time -= (*hour) * SECS_PER_HOUR;
-	*min = time / SECS_PER_MINUTE;
-	time -= (*min) * SECS_PER_MINUTE;
-	*sec = time;
-	*fsec = time - *sec;
-#endif
-}	/* dt2time() */
-
 /* timestamp2tm()
  * Convert timestamp data type to POSIX time structure.
  * Note that year is _not_ 1900-based, but is an explicit full value.
@@ -308,7 +289,6 @@ PGTYPEStimestamp_from_asc(char *str, char **endptr)
 	fsec_t		fsec;
 	struct tm	tt,
 			   *tm = &tt;
-	int			tz;
 	int			dtype;
 	int			nf;
 	char	   *field[MAXDATEFIELDS];
@@ -323,8 +303,8 @@ PGTYPEStimestamp_from_asc(char *str, char **endptr)
 		return (noresult);
 	}
 
-	if (ParseDateTime(str, lowstr, field, ftype, MAXDATEFIELDS, &nf, ptr) != 0 ||
-		DecodeDateTime(field, ftype, nf, &dtype, tm, &fsec, &tz, 0) != 0)
+	if (ParseDateTime(str, lowstr, field, ftype, &nf, ptr) != 0 ||
+		DecodeDateTime(field, ftype, nf, &dtype, tm, &fsec, 0) != 0)
 	{
 		errno = PGTYPES_TS_BAD_TIMESTAMP;
 		return (noresult);
@@ -400,7 +380,8 @@ PGTYPEStimestamp_current(timestamp * ts)
 	struct tm	tm;
 
 	GetCurrentDateTime(&tm);
-	tm2timestamp(&tm, 0, NULL, ts);
+	if (errno == 0)
+		tm2timestamp(&tm, 0, NULL, ts);
 	return;
 }
 
@@ -423,34 +404,50 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 			replace_type = PGTYPES_TYPE_NOTHING;
 			switch (*p)
 			{
+					/* the abbreviated name of the day in the week */
+					/* XXX should be locale aware */
 				case 'a':
 					replace_val.str_val = pgtypes_date_weekdays_short[dow];
 					replace_type = PGTYPES_TYPE_STRING_CONSTANT;
 					break;
+					/* the full name of the day in the week */
+					/* XXX should be locale aware */
 				case 'A':
 					replace_val.str_val = days[dow];
 					replace_type = PGTYPES_TYPE_STRING_CONSTANT;
 					break;
+					/* the abbreviated name of the month */
+					/* XXX should be locale aware */
 				case 'b':
 				case 'h':
 					replace_val.str_val = months[tm->tm_mon];
 					replace_type = PGTYPES_TYPE_STRING_CONSTANT;
 					break;
+					/* the full name name of the month */
+					/* XXX should be locale aware */
 				case 'B':
 					replace_val.str_val = pgtypes_date_months[tm->tm_mon];
 					replace_type = PGTYPES_TYPE_STRING_CONSTANT;
 					break;
+
+					/*
+					 * The	preferred  date  and  time	representation	for
+					 * the current locale.
+					 */
 				case 'c':
 					/* XXX */
 					break;
+					/* the century number with leading zeroes */
 				case 'C':
 					replace_val.uint_val = tm->tm_year / 100;
 					replace_type = PGTYPES_TYPE_UINT_2_LZ;
 					break;
+					/* day with leading zeroes (01 - 31) */
 				case 'd':
 					replace_val.uint_val = tm->tm_mday;
 					replace_type = PGTYPES_TYPE_UINT_2_LZ;
 					break;
+					/* the date in the format mm/dd/yy */
 				case 'D':
 
 					/*
@@ -467,10 +464,15 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					if (i)
 						return i;
 					break;
+					/* day with leading spaces (01 - 31) */
 				case 'e':
 					replace_val.uint_val = tm->tm_mday;
 					replace_type = PGTYPES_TYPE_UINT_2_LS;
 					break;
+
+					/*
+					 * alternative format modifier
+					 */
 				case 'E':
 					{
 						char		tmp[4] = "%Ex";
@@ -479,7 +481,7 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 						if (*p == '\0')
 							return -1;
 						tmp[2] = *p;
-						
+
 						/*
 						 * strftime's month is 0 based, ours is 1 based
 						 */
@@ -496,6 +498,11 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 						replace_type = PGTYPES_TYPE_NOTHING;
 						break;
 					}
+
+					/*
+					 * The ISO 8601 year with century as a decimal number. The
+					 * 4-digit year corresponding to the ISO week number.
+					 */
 				case 'G':
 					tm->tm_mon -= 1;
 					i = strftime(q, *pstr_len, "%G", tm);
@@ -509,9 +516,14 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					tm->tm_mon += 1;
 					replace_type = PGTYPES_TYPE_NOTHING;
 					break;
+
+					/*
+					 * Like %G, but without century, i.e., with a 2-digit year
+					 * (00-99).
+					 */
 				case 'g':
 					{
-						char	   *fmt = "%g"; /* Keep compiler quiet about
+						const char *fmt = "%g"; /* Keep compiler quiet about
 												 * 2-digit year */
 
 						tm->tm_mon -= 1;
@@ -527,38 +539,60 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 						replace_type = PGTYPES_TYPE_NOTHING;
 					}
 					break;
+					/* hour (24 hour clock) with leading zeroes */
 				case 'H':
 					replace_val.uint_val = tm->tm_hour;
 					replace_type = PGTYPES_TYPE_UINT_2_LZ;
 					break;
+					/* hour (12 hour clock) with leading zeroes */
 				case 'I':
 					replace_val.uint_val = tm->tm_hour % 12;
 					replace_type = PGTYPES_TYPE_UINT_2_LZ;
 					break;
+
+					/*
+					 * The day of the year as a decimal number with leading
+					 * zeroes. It ranges from 001 to 366.
+					 */
 				case 'j':
 					replace_val.uint_val = tm->tm_yday;
 					replace_type = PGTYPES_TYPE_UINT_3_LZ;
 					break;
+
+					/*
+					 * The hour (24 hour clock). Leading zeroes will be turned
+					 * into spaces.
+					 */
 				case 'k':
 					replace_val.uint_val = tm->tm_hour;
 					replace_type = PGTYPES_TYPE_UINT_2_LS;
 					break;
+
+					/*
+					 * The hour (12 hour clock). Leading zeroes will be turned
+					 * into spaces.
+					 */
 				case 'l':
 					replace_val.uint_val = tm->tm_hour % 12;
 					replace_type = PGTYPES_TYPE_UINT_2_LS;
 					break;
+					/* The month as a decimal number with a leading zero */
 				case 'm':
 					replace_val.uint_val = tm->tm_mon;
 					replace_type = PGTYPES_TYPE_UINT_2_LZ;
 					break;
+					/* The minute as a decimal number with a leading zero */
 				case 'M':
 					replace_val.uint_val = tm->tm_min;
 					replace_type = PGTYPES_TYPE_UINT_2_LZ;
 					break;
+					/* A newline character */
 				case 'n':
 					replace_val.char_val = '\n';
 					replace_type = PGTYPES_TYPE_CHAR;
 					break;
+					/* the AM/PM specifier (uppercase) */
+					/* XXX should be locale aware */
 				case 'p':
 					if (tm->tm_hour < 12)
 						replace_val.str_val = "AM";
@@ -566,6 +600,8 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 						replace_val.str_val = "PM";
 					replace_type = PGTYPES_TYPE_STRING_CONSTANT;
 					break;
+					/* the AM/PM specifier (lowercase) */
+					/* XXX should be locale aware */
 				case 'P':
 					if (tm->tm_hour < 12)
 						replace_val.str_val = "am";
@@ -573,6 +609,8 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 						replace_val.str_val = "pm";
 					replace_type = PGTYPES_TYPE_STRING_CONSTANT;
 					break;
+					/* the time in the format %I:%M:%S %p */
+					/* XXX should be locale aware */
 				case 'r':
 					i = dttofmtasc_replace(ts, dDate, dow, tm,
 										   q, pstr_len,
@@ -580,6 +618,7 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					if (i)
 						return i;
 					break;
+					/* The time in 24 hour notation (%H:%M) */
 				case 'R':
 					i = dttofmtasc_replace(ts, dDate, dow, tm,
 										   q, pstr_len,
@@ -587,6 +626,7 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					if (i)
 						return i;
 					break;
+					/* The number of seconds since the Epoch (1970-01-01) */
 				case 's':
 #ifdef HAVE_INT64_TIMESTAMP
 					replace_val.int64_val = (*ts - SetEpochTimestamp()) / 1000000.0;
@@ -596,14 +636,17 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					replace_type = PGTYPES_TYPE_DOUBLE_NF;
 #endif
 					break;
+					/* seconds as a decimal number with leading zeroes */
 				case 'S':
 					replace_val.uint_val = tm->tm_sec;
 					replace_type = PGTYPES_TYPE_UINT_2_LZ;
 					break;
+					/* A tabulator */
 				case 't':
 					replace_val.char_val = '\t';
 					replace_type = PGTYPES_TYPE_CHAR;
 					break;
+					/* The time in 24 hour notation (%H:%M:%S) */
 				case 'T':
 					i = dttofmtasc_replace(ts, dDate, dow, tm,
 										   q, pstr_len,
@@ -611,12 +654,18 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					if (i)
 						return i;
 					break;
+
+					/*
+					 * The day of the week as a decimal, Monday = 1, Sunday =
+					 * 7
+					 */
 				case 'u':
-					if (dow == 0)
-						dow = 7;
 					replace_val.uint_val = dow;
+					if (replace_val.uint_val == 0)
+						replace_val.uint_val = 7;
 					replace_type = PGTYPES_TYPE_UINT;
 					break;
+					/* The week number of the year as a decimal number */
 				case 'U':
 					tm->tm_mon -= 1;
 					i = strftime(q, *pstr_len, "%U", tm);
@@ -630,6 +679,11 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					tm->tm_mon += 1;
 					replace_type = PGTYPES_TYPE_NOTHING;
 					break;
+
+					/*
+					 * The ISO 8601:1988 week number of the current year as a
+					 * decimal number.
+					 */
 				case 'V':
 					i = strftime(q, *pstr_len, "%V", tm);
 					if (i == 0)
@@ -641,10 +695,16 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					}
 					replace_type = PGTYPES_TYPE_NOTHING;
 					break;
+
+					/*
+					 * The day of the week as a decimal, Sunday being 0 and
+					 * Monday 1.
+					 */
 				case 'w':
 					replace_val.uint_val = dow;
 					replace_type = PGTYPES_TYPE_UINT;
 					break;
+					/* The week number of the year (another definition) */
 				case 'W':
 					tm->tm_mon -= 1;
 					i = strftime(q, *pstr_len, "%U", tm);
@@ -658,9 +718,14 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					tm->tm_mon += 1;
 					replace_type = PGTYPES_TYPE_NOTHING;
 					break;
+
+					/*
+					 * The preferred date representation for the current
+					 * locale without the time.
+					 */
 				case 'x':
 					{
-						char	   *fmt = "%x"; /* Keep compiler quiet about
+						const char *fmt = "%x"; /* Keep compiler quiet about
 												 * 2-digit year */
 
 						tm->tm_mon -= 1;
@@ -676,6 +741,11 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 						replace_type = PGTYPES_TYPE_NOTHING;
 					}
 					break;
+
+					/*
+					 * The preferred time representation for the current
+					 * locale without the date.
+					 */
 				case 'X':
 					tm->tm_mon -= 1;
 					i = strftime(q, *pstr_len, "%X", tm);
@@ -689,14 +759,17 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					tm->tm_mon += 1;
 					replace_type = PGTYPES_TYPE_NOTHING;
 					break;
+					/* The year without the century (2 digits, leading zeroes) */
 				case 'y':
 					replace_val.uint_val = tm->tm_year % 100;
 					replace_type = PGTYPES_TYPE_UINT_2_LZ;
 					break;
+					/* The year with the century (4 digits) */
 				case 'Y':
 					replace_val.uint_val = tm->tm_year;
 					replace_type = PGTYPES_TYPE_UINT;
 					break;
+					/* The time zone offset from GMT */
 				case 'z':
 					tm->tm_mon -= 1;
 					i = strftime(q, *pstr_len, "%z", tm);
@@ -710,6 +783,7 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					tm->tm_mon += 1;
 					replace_type = PGTYPES_TYPE_NOTHING;
 					break;
+					/* The name or abbreviation of the time zone */
 				case 'Z':
 					tm->tm_mon -= 1;
 					i = strftime(q, *pstr_len, "%Z", tm);
@@ -723,12 +797,13 @@ dttofmtasc_replace(timestamp * ts, date dDate, int dow, struct tm * tm,
 					tm->tm_mon += 1;
 					replace_type = PGTYPES_TYPE_NOTHING;
 					break;
+					/* A % sign */
 				case '%':
 					replace_val.char_val = '%';
 					replace_type = PGTYPES_TYPE_CHAR;
 					break;
 				case '\0':
-					/* fmtstr: blabla%' */
+					/* fmtstr: foo%' - The string ends with a % sign */
 
 					/*
 					 * this is not compliant to the specification
